"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.metricNames = exports.hamming = exports.canberra = exports.chebyshev = exports.braycurtis = exports.manhattan = exports.euclidean = void 0;
const uf = __importStar(require("./utils"));
const euclidean = (v1, v2) => {
    return Math.sqrt(v1
        .map((el, i) => {
        return Math.pow(el - v2[i], 2);
    })
        .reduce(uf.psum, 0));
};
exports.euclidean = euclidean;
const manhattan = (v1, v2) => {
    return v1
        .map((el, i) => {
        return Math.abs(el - v2[i]);
    })
        .reduce(uf.psum);
};
exports.manhattan = manhattan;
const braycurtis = (v1, v2) => {
    const n = v1
        .map((el, i) => {
        return Math.abs(el - v2[i]);
    })
        .reduce(uf.psum);
    const d = uf.asum(v1) + uf.asum(v2);
    return n / d;
};
exports.braycurtis = braycurtis;
const chebyshev = (v1, v2) => {
    return v1
        .map((el, i) => {
        return Math.abs(el - v2[i]);
    })
        .reduce(uf.pmax);
};
exports.chebyshev = chebyshev;
const canberra = (v1, v2) => {
    return v1
        .reduce(function (acc, el, i) {
        const n = Math.abs(el - v2[i]);
        const d = Math.abs(el) + Math.abs(v2[i]);
        if (d !== 0) {
            acc.push(n / d);
        }
        return acc;
    }, [])
        .reduce(uf.psum);
};
exports.canberra = canberra;
const hamming = (v1, v2) => {
    var cnts = v1.reduce(function (acc, el, i) {
        let si = el !== v2[i] ? 1 : 0;
        acc.s += si;
        acc.t += 1;
        return acc;
    }, { s: 0, t: 0 });
    return cnts.s / cnts.t;
};
exports.hamming = hamming;
exports.metricNames = [
    'euclidean',
    'manhattan',
    'braycurtis',
    'chebyshev',
    'canberra',
    'hamming',
];
